<?php

namespace Drupal\communication\Plugin\Field\FieldType;

use Drupal\communication\Contact\ContactInfo;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\Field\Plugin\Field\FieldType\EntityReferenceItem;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\TypedData\DataDefinition;

/**
 * Class ContactReferenceItem
 *
 * @FieldType(
 *   id = "contact_reference",
 *   label = @Translation("Contact Reference"),
 *   description = @Translation("A reference field to contacts"),
 *   category = @Translation("Reference"),
 *   default_widget = "contact_reference_autocomplete",
 *   default_formatter = "contact_reference_default",
 *   list_class = "\Drupal\Core\Field\EntityReferenceFieldItemList"
 * )
 *
 * @package Drupal\communication\Plugin\Field\FieldType
 */
class ContactReferenceItem extends EntityReferenceItem {

  /**
   * {@inheritdoc}
   */
  public static function defaultStorageSettings() {
    $settings = parent::defaultStorageSettings();
    $settings['target_type'] = \Drupal::hasService('communication.contact_middleware') ? \Drupal::service('communication.contact_middleware')->contactEntityTypeId() : 'user';

    if (\Drupal::hasService('communication.contact_middleware')) {
      $settings['handler'] = \Drupal::service('communication.contact_middleware')->contactSelectionHandler();
    }

    return $settings;
  }

  /**
   * {@inheritdoc}
   */
  public static function schema(FieldStorageDefinitionInterface $field_definition) {
    $schema = parent::schema($field_definition);

    $schema['key'] = [
      'type' => 'varchar',
      'length' => 255,
      'not null' => TRUE,
      'default' => '',
    ];

    return $schema;
  }

  public static function propertyDefinitions(FieldStorageDefinitionInterface $field_definition) {
    $properties = parent::propertyDefinitions($field_definition);

    $properties['key'] = DataDefinition::create('string')
      ->setLabel(new TranslatableMarkup('Key'));

    $properties['info'] = DataDefinition::create('contact_info')
      ->setLabel(new TranslatableMarkup('Contact Info'))
      ->setComputed(TRUE);

    return $properties;
  }

  public function preSave() {
    parent::preSave(); // TODO: Change the autogenerated stub

    // Attempt to write back participant data where appropriate.
    if (($contact = $this->entity) && ($key = $this->key)) {
      $contact_info = ContactInfo::create($contact, $key);

      if (!empty($this->values['metadata'])) {
        $write_back_values['metadata'] = $this->values['metadata'];
      }
      foreach ($this->getEntity()->getFieldDefinitions() as $field_name => $definition) {
        if (in_array($field_name, [
          'id',
          'vid',
          'revision_default',
          'uuid',
          'type',
          'contact',
          'role',
          'communication'
        ])) {
          continue;
        }

        if ($contact_info->supportsWriteBackInfoValue($field_name, $definition)) {
          $write_back_values[$field_name] = $this->getEntity()->get($field_name)->getValue();
        }
      }

      if (!empty($write_back_values)) {
        $contact_info->writeBackInfoValues($definition, $write_back_values);
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function setValue($values, $notify = TRUE) {
    parent::setValue($values, FALSE);

    if (is_array($values) && !empty($values['key']) && (isset($values['target_id']) || isset($values['entity']))) {
      $contact = $this->get('entity');
      $this->writePropertyValue('info', ContactInfo::create($contact->getValue(), $values['key']));
    }

    // Notify the parent if necessary.
    if ($notify && $this->parent) {
      $this->parent->onChange($this->getName());
    }
  }
}
